#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016-07-31 
#
# Monitor Check Point management server
#
# 2017-03-08: changed snmp scan and inventory function
# 2018-03-15: code cleanup
# 2020-07-08: changed snm-scan function
# 2021-08-10: rewritten for CMK 2.0
# 2021-08-11: cleanup, removed unused OIDs, added WATO
# 2023-05-29: moved gui files to ~/local/lib7check_mk/gui/plugins/...
#
# sample snmpwalk (client table missing)
#
# .1.3.6.1.4.1.2620.1.7.1.0 = STRING: Check Point Security Management Server
# .1.3.6.1.4.1.2620.1.7.2.0 = INTEGER: 6
# .1.3.6.1.4.1.2620.1.7.3.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.7.4.0 = Wrong Type (should be INTEGER): Gauge32: 991140003
# .1.3.6.1.4.1.2620.1.7.5.0 = STRING: active
# .1.3.6.1.4.1.2620.1.7.6.0 = INTEGER: 1
# .1.3.6.1.4.1.2620.1.7.9.0 = STRING: MgmtHAJournals is too long.
# .1.3.6.1.4.1.2620.1.7.10.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.7.11.0 = STRING: License violation detected:
# .1.3.6.1.4.1.2620.1.7.101.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.7.102.0 = STRING: OK
# .1.3.6.1.4.1.2620.1.7.103.0 = STRING: OK
#
#
# sample string_table
#
# [['6', '0', '996000220', 'active', '1', '0', '', '0', 'OK', 'OK']]
#
# no MGMT Server
# [[]]
#
# sample section
#
# CheckpointMgmt(version='6.0', buildnumber='996000220', mode='active', fwmisalive=1, islicenseviolation=0,
#                licenseviolationmsg='', statcode=0, statshortdescr='OK', statlongdescr='OK')
#

from typing import Optional
from dataclasses import dataclass

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    equals,
    Result,
    State,
    SNMPTree,
    startswith,
    all_of,
    any_of,
)


@dataclass
class CheckpointMgmt:
    version: str
    buildnumber: str
    mode: str
    fwmisalive: int
    islicenseviolation: int
    licenseviolationmsg: str
    statcode: int
    statshortdescr: str
    statlongdescr: str


def parse_checkpoint_mgmt(string_table: StringTable) -> Optional[CheckpointMgmt]:
    try:
        vermajor, verminor, buildnumber, mode, fwmisalive, islicenseviolation, licenseviolationmsg, \
        statcode, statshortdescr, statlongdescr = string_table[0]
    except (IndexError, ValueError):
        return

    return CheckpointMgmt(
        version=f'{vermajor}.{verminor}',
        buildnumber=buildnumber,
        mode=mode,
        fwmisalive=int(fwmisalive),
        islicenseviolation=int(islicenseviolation),
        licenseviolationmsg=licenseviolationmsg,
        statcode=int(statcode),
        statshortdescr=statshortdescr,
        statlongdescr=statlongdescr,
    )


def discovery_checkpoint_mgmt(section: CheckpointMgmt) -> DiscoveryResult:
    yield Service()


def check_checkpoint_mgmt(params, section: CheckpointMgmt) -> CheckResult:
    yield Result(state=State.OK, summary=f'Version: {section.version}')
    yield Result(state=State.OK, summary=f'Build: {section.buildnumber}')

    if section.mode.lower() != params['desired_mode']:
        yield Result(state=State(params['state_if_not_desired']), summary=f'Mode: {section.mode}')
    else:
        yield Result(state=State.OK, summary=f'Mode: {section.mode}')

    if section.islicenseviolation != 0:
        if len(section.licenseviolationmsg) != 0:
            yield Result(state=State(params['state_islicenseviolation']), notice=section.licenseviolationmsg)
        else:
            yield Result(state=State(params['state_islicenseviolation']), notice='license violation')

    if section.fwmisalive != 1:
        yield Result(state=State(params['state_not_fwmisalive']), notice='is not alive')

    if section.statcode != 0:
        yield Result(state=State(params['state_statcode']),
                     notice=f'Status : {section.statshortdescr}, {section.statlongdescr}')


register.snmp_section(
    name='checkpoint_mgmt',
    parse_function=parse_checkpoint_mgmt,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.1.7',  # CHECKPOINT-MIB::mngmt
        oids=[
            '2',  # mgVerMajor
            '3',  # mgVerMinor
            '4',  # mgBuildNumber
            '5',  # mgActiveStatus
            '6',  # mgFwmIsAlive
            '10',  # mgIsLicenseViolation
            '11',  # mgLicenseViolationMsg
            '101',  # mgStatCode
            '102',  # mgStatShortDescr
            '103',  # mgStatLongDescr
        ]),
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_mgmt',
    service_name='Management server',
    discovery_function=discovery_checkpoint_mgmt,
    check_function=check_checkpoint_mgmt,
    check_ruleset_name='checkpoint_mgmt',
    check_default_parameters={
        'desired_mode': 'active',
        'state_if_not_desired': 1,
        'state_islicenseviolation': 1,
        'state_not_fwmisalive': 2,
        'state_statcode': 2,
    },
)
